<?php
/**
 * Demo Deployment Tool
 *
 * @version		1.2.0
 * @author		Martin Rasser, Elovaris Web Development
 * @copyright	Copyright (C) 2009 Elovaris Web Development. All rights reserved.
 * 				Borrows some code from the Joomla! project (Password generation)
 * @license		GNU/GPL (http://www.gnu.org/licenses/gpl-2.0.html)
 */
defined('PROPER_ENTRY') or die('Direct access to this file not allowed');
/**
 * @version		1.1
 * @package		Joomla Demo Site Deployer
 * 
 * Borrowed code from Joomla! 
 */
 
function getCryptedPassword($plaintext, $salt = '', $encryption = 'md5-hex', $show_encrypt = false) {
	// Get the salt to use.
	$salt = getSalt($encryption, $salt, $plaintext);

	// Encrypt the password.
	switch ($encryption)
	{
		case 'plain' :
			return $plaintext;

		case 'sha' :
			$encrypted = base64_encode(mhash(MHASH_SHA1, $plaintext));
			return ($show_encrypt) ? '{SHA}'.$encrypted : $encrypted;

		case 'crypt' :
		case 'crypt-des' :
		case 'crypt-md5' :
		case 'crypt-blowfish' :
			return ($show_encrypt ? '{crypt}' : '').crypt($plaintext, $salt);

		case 'md5-base64' :
			$encrypted = base64_encode(mhash(MHASH_MD5, $plaintext));
			return ($show_encrypt) ? '{MD5}'.$encrypted : $encrypted;

		case 'ssha' :
			$encrypted = base64_encode(mhash(MHASH_SHA1, $plaintext.$salt).$salt);
			return ($show_encrypt) ? '{SSHA}'.$encrypted : $encrypted;

		case 'smd5' :
			$encrypted = base64_encode(mhash(MHASH_MD5, $plaintext.$salt).$salt);
			return ($show_encrypt) ? '{SMD5}'.$encrypted : $encrypted;

		case 'aprmd5' :
			$length = strlen($plaintext);
			$context = $plaintext.'$apr1$'.$salt;
			$binary = _bin(md5($plaintext.$salt.$plaintext));

			for ($i = $length; $i > 0; $i -= 16) {
				$context .= substr($binary, 0, ($i > 16 ? 16 : $i));
			}
			for ($i = $length; $i > 0; $i >>= 1) {
				$context .= ($i & 1) ? chr(0) : $plaintext[0];
			}

			$binary = _bin(md5($context));

			for ($i = 0; $i < 1000; $i ++) {
				$new = ($i & 1) ? $plaintext : substr($binary, 0, 16);
				if ($i % 3) {
					$new .= $salt;
				}
				if ($i % 7) {
					$new .= $plaintext;
				}
				$new .= ($i & 1) ? substr($binary, 0, 16) : $plaintext;
				$binary = _bin(md5($new));
			}

			$p = array ();
			for ($i = 0; $i < 5; $i ++) {
				$k = $i +6;
				$j = $i +12;
				if ($j == 16) {
					$j = 5;
				}
				$p[] = _toAPRMD5((ord($binary[$i]) << 16) | (ord($binary[$k]) << 8) | (ord($binary[$j])), 5);
			}

			return '$apr1$'.$salt.'$'.implode('', $p)._toAPRMD5(ord($binary[11]), 3);

		case 'md5-hex' :
		default :
			$encrypted = ($salt) ? md5($plaintext.$salt) : md5($plaintext);
			return ($show_encrypt) ? '{MD5}'.$encrypted : $encrypted;
	}
}

function getSalt($encryption = 'md5-hex', $seed = '', $plaintext = '') {
	// Encrypt the password.
	switch ($encryption)
	{
		case 'crypt' :
		case 'crypt-des' :
			if ($seed) {
				return substr(preg_replace('|^{crypt}|i', '', $seed), 0, 2);
			} else {
				return substr(md5(mt_rand()), 0, 2);
			}
			break;

		case 'crypt-md5' :
			if ($seed) {
				return substr(preg_replace('|^{crypt}|i', '', $seed), 0, 12);
			} else {
				return '$1$'.substr(md5(mt_rand()), 0, 8).'$';
			}
			break;

		case 'crypt-blowfish' :
			if ($seed) {
				return substr(preg_replace('|^{crypt}|i', '', $seed), 0, 16);
			} else {
				return '$2$'.substr(md5(mt_rand()), 0, 12).'$';
			}
			break;

		case 'ssha' :
			if ($seed) {
				return substr(preg_replace('|^{SSHA}|', '', $seed), -20);
			} else {
				return mhash_keygen_s2k(MHASH_SHA1, $plaintext, substr(pack('h*', md5(mt_rand())), 0, 8), 4);
			}
			break;

		case 'smd5' :
			if ($seed) {
				return substr(preg_replace('|^{SMD5}|', '', $seed), -16);
			} else {
				return mhash_keygen_s2k(MHASH_MD5, $plaintext, substr(pack('h*', md5(mt_rand())), 0, 8), 4);
			}
			break;

		case 'aprmd5' :
			/* 64 characters that are valid for APRMD5 passwords. */
			$APRMD5 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

			if ($seed) {
				return substr(preg_replace('/^\$apr1\$(.{8}).*/', '\\1', $seed), 0, 8);
			} else {
				$salt = '';
				for ($i = 0; $i < 8; $i ++) {
					$salt .= $APRMD5 {
						rand(0, 63)
						};
				}
				return $salt;
			}
			break;

		default :
			$salt = '';
			if ($seed) {
				$salt = $seed;
			}
			return $salt;
			break;
	}
}


/**
 * Converts hexadecimal string to binary data.
 *
 * @access private
 * @param string $hex  Hex data.
 * @return string  Binary data.
 * @since 1.5
 */
function _bin($hex)
{
	$bin = '';
	$length = strlen($hex);
	for ($i = 0; $i < $length; $i += 2) {
		$tmp = sscanf(substr($hex, $i, 2), '%x');
		$bin .= chr(array_shift($tmp));
	}
	return $bin;
}

/**
 * Converts to allowed 64 characters for APRMD5 passwords.
 *
 * @access private
 * @param string  $value
 * @param integer $count
 * @return string  $value converted to the 64 MD5 characters.
 * @since 1.5
 */
function _toAPRMD5($value, $count)
{
	/* 64 characters that are valid for APRMD5 passwords. */
	$APRMD5 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

	$aprmd5 = '';
	$count = abs($count);
	while (-- $count) {
		$aprmd5 .= $APRMD5[$value & 0x3f];
		$value >>= 6;
	}
	return $aprmd5;
}

/**
 * Generate a random password
 *
 * @static
 * @param	int		$length	Length of the password to generate
 * @return	string			Random Password
 * @since	1.5
 */
function genRandomPassword($length = 8)
{
	$salt = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
	$len = strlen($salt);
	$makepass = '';

	$stat = @stat(__FILE__);
	if(empty($stat) || !is_array($stat)) $stat = array(php_uname());

	mt_srand(crc32(microtime() . implode('|', $stat)));

	for ($i = 0; $i < $length; $i ++) {
		$makepass .= $salt[mt_rand(0, $len -1)];
	}

	return $makepass;
}

function mb_wordwrap($str, $width=74, $break="\r\n")
{
    // Return short or empty strings untouched
    if(empty($str) || mb_strlen($str, 'UTF-8') <= $width)
        return $str;
   
    $br_width  = mb_strlen($break, 'UTF-8');
    $str_width = mb_strlen($str, 'UTF-8');
    $return = '';
    $last_space = false;
    
    for($i=0, $count=0; $i < $str_width; $i++, $count++)
    {
        // If we're at a break
        if (mb_substr($str, $i, $br_width, 'UTF-8') == $break)
        {
            $count = 0;
            $return .= mb_substr($str, $i, $br_width, 'UTF-8');
            $i += $br_width - 1;
            continue;
        }

        // Keep a track of the most recent possible break point
        /*
        if(mb_substr($str, $i, 1, 'UTF-8') == " ")
        {
            $last_space = $i;
        }
        */

        // It's time to wrap
        if ($count > $width)
        {
            // There are no spaces to break on!  Going to truncate :(
            if(!$last_space)
            {
                $return .= $break;
                $count = 0;
            }
            else
            {
                // Work out how far back the last space was
                $drop = $i - $last_space;

                // Cutting zero chars results in an empty string, so don't do that
                if($drop > 0)
                {
                    $return = mb_substr($return, 0, -$drop);
                }
                
                // Add a break
                $return .= $break;

                // Update pointers
                $i = $last_space + ($br_width - 1);
                $last_space = false;
                $count = 0;
            }
        }

        // Add character from the input string to the output
        $return .= mb_substr($str, $i, 1, 'UTF-8');
    }
    return $return;
}

?>